from dataclasses import dataclass


@dataclass(frozen=True)
class CHAIN_SCAN_API:
    ETH: str = 'https://api.etherscan.io/api'
    ETH_CN: str = 'https://api-cn.etherscan.com/api'
    BSC: str = 'https://api.bscscan.com/api'
    HECO: str = 'https://api.hecoinfo.com/api'
    MATIC: str = 'https://api.polygonscan.com/api'
    FTM: str = 'https://api.ftmscan.com/api'
    MOONRIVER: str = 'https://api-moonriver.moonscan.io/api'
    AVAX: str = 'https://api.snowtrace.io/api'  # showtrace
    KCC: str = ''  # not supported
    ARBITRUM: str = 'https://api.arbiscan.io/'
    CRONOS: str = 'https://api.cronoscan.com/api'


@dataclass(frozen=True)
class CHAIN_CONTRACT_API:
    ETH: str = 'https://api.etherscan.io/api?module=contract&action=getabi&address={contract_address}&apikey={api_key}'
    ETH_CN: str = 'https://api-cn.etherscan.com/api?module=contract&action=getabi&address={contract_address}&apikey={api_key}'
    BSC: str = 'https://api.bscscan.com/api?module=contract&action=getabi&address={contract_address}&apikey={api_key}'
    HECO: str = 'https://api.hecoinfo.com/api?module=contract&action=getabi&address={contract_address}&apikey={api_key}'
    MATIC: str = 'https://api.polygonscan.com/api?module=contract&action=getabi&address={contract_address}&apikey={api_key}'
    FTM: str = 'https://api.ftmscan.com/api?module=contract&action=getabi&address={contract_address}&apikey={api_key}'
    MOONRIVER: str = 'https://api-moonriver.moonscan.io/api?module=contract&action=getabi&address={contract_address}&apikey={api_key}'
    # showtrace
    AVAX: str = 'https://api.snowtrace.io/api?module=contract&action=getabi&address={contract_address}&apikey={api_key}'
    KCC: str = ''  # not support
    ARBITRUM: str = 'https://api.arbiscan.io/api?module=contract&action=getabi&address={contract_address}&apikey={api_key}'
    CRONOS: str = 'https://api.cronoscan.com/api?module=contract&action=getabi&address={contract_address}&apikey={api_key}'



import json
import os
import glob
from types import SimpleNamespace

root_path = f"{os.path.dirname(os.path.abspath(__file__))}/assets"


def _load_json_file(fp):
    with open(fp) as f:
        abi = json.load(f)
    return abi


def load_abi(name: str):
    with open(os.path.abspath(os.path.join(root_path, f'{name}.abi'))) as f:
        abi = json.load(f)
    return abi


def load_all_abis():
    all_abis_fp = list(glob.iglob(f"{root_path}/**/*.abi", recursive=True))
    return {f'{p[len(root_path):][:-4]}': _load_json_file(p) for p in all_abis_fp}


def load_chains():
    fp = os.path.join(root_path, "utils", "chains.json")
    return _load_json_file(fp)



from setuptools import setup, find_packages

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()
# print(find_packages())
setup(
    name='many_abis',
    packages=['many_abis'],
    package_dir={'many_abis': 'many_abis'},
    package_data={'': [
        '**/*.abi', '**/*.json']},
    version='0.1.5',
    license='MIT',
    description='A simple way to get different DEXs abis for block chains.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Yong',
    author_email='ackness8@gmail.com',
    url='https://github.com/ackness/many_abis',
    keywords=['abi', 'dex', 'block chain', 'bsc', 'eth', 'matic', 'heco',
              'kcc', 'pancakeswap', 'mdex', 'quickswap', 'uniswap', 'koffeeswap'],

    install_requires=[
        "eth_utils",
        "addict"
    ],

)



from .utils import load_chains
from addict import Dict
from functools import reduce

CHAINS = Dict(load_chains())
SUPPORTED_CHAINS = list(CHAINS.keys())


def all_chains():
    return SUPPORTED_CHAINS


def get_chain_by_name(name: str):
    assert name in SUPPORTED_CHAINS, f"Chain {name} not supported"
    return CHAINS[name]


def get_chain_by_id(chain_id: int):
    for name, info in CHAINS.items():
        if "chain_id" not in info:
            raise ValueError(f"Chain {name} does not have chain_id")
        if info["chain_id"] == chain_id:
            return CHAINS[name]
    raise ValueError(f"Chain id {chain_id} not supported")


def get_chain(name=None, chain_id=None):
    assert name or chain_id, "Must provide either name or chain_id"
    if name:
        name = name.lower()
        return get_chain_by_name(name)
    elif chain_id:
        chain_id = int(chain_id)
        return get_chain_by_id(chain_id)


def chain(*args, **kwargs):
    return get_chain(*args, **kwargs)


def get(*keys):
    try:
        return reduce(lambda d, key: d[key], keys, CHAINS)
    except KeyError as e:
        raise KeyError(f"{keys} is not exist") from e



from .utils import load_all_abis
from addict import Dict
import requests


def all_abis():
    all_abis = load_all_abis()
    names = []
    all_abis_rename = {}
    for k, v in all_abis.items():
        ns = k.split('/')[2:]
        name = '_'.join(ns).upper()
        names.append(name)
        all_abis_rename[name] = v
    return names, Dict(all_abis_rename)


ALL_ABIS_NAME, ABIS = all_abis()
globals().update(ABIS)


def get_abi_from_address(address: str, api_key: str, chain_api: str):
    '''
    address: Address want to get abi
    api_key: Explorer API Key
    chain_api: api format for get abi, format like this,
        'https://xxx.xxxscan.xxx/api?module=contract&action=getabi&address={contract_address}&apikey={api_key}'
    '''
    try:
        query = chain_api.format(contract_address=address, api_key=api_key)
        # print(query)
        session = requests.Session()
        headers = {
            'User-Agent': 'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3)'
        }
        result = session.get(query, headers=headers)
        session.close()
        result = eval(result.content.decode())
        # print(result)
        if result['status'] == '1' and result['message'] == 'OK':
            return result['result']
        else:
            return None
    except Exception as e:
        print(e)
        return None
